#include "LCDPlaybackManager.h"
#include "resource.h"

LCDPlaybackManager::LCDPlaybackManager()
{
	iLeftTimeFormat = 0;
	iRightTimeFormat = 0;

	strcpy(m_szTrackPosition, "0:00");
	strcpy(m_szTrackCountDown, "?:??");
}


HRESULT LCDPlaybackManager::Initialize(void)
{
	m_Line1.Initialize();
	m_Line1.SetOrigin(0, 0);
    m_Line1.SetSize(160, 11);	
	m_Line1.SetFontFaceName(_T("Small Fonts"));
	m_Line1.SetFontPointSize(7);
    m_Line1.SetAlignment(DT_CENTER);
	m_Line1.SetSpeed(20);
	m_Line1.SetStartDelay(1000);
    m_Line1.SetText(_T(" "));

	m_Line2.Initialize();
	m_Line2.SetOrigin(0, 11);
    m_Line2.SetSize(160, 11);
	m_Line2.SetFontFaceName(_T("Small Fonts"));
	m_Line2.SetFontPointSize(7);
    m_Line2.SetAlignment(DT_CENTER);
	m_Line2.SetSpeed(20);
	m_Line2.SetStartDelay(1000);
    m_Line2.SetText(_T(" "));	

	m_Line3.Initialize();
	m_Line3.SetOrigin(0, 22);
	m_Line3.SetSize(160, 11);
	m_Line3.SetFontFaceName(_T("Small Fonts"));
	m_Line3.SetFontPointSize(7);
    m_Line3.SetAlignment(DT_CENTER);
	m_Line3.SetSpeed(20);
	m_Line3.SetStartDelay(1000);
    m_Line3.SetText(_T(" "));	

	m_ProgressBar.Initialize();
	m_ProgressBar.SetOrigin(0, 39);
	m_ProgressBar.SetSize(160, 4);
	m_ProgressBar.SetRange(0, 100);
	m_ProgressBar.SetPos(0);

	m_TimeLeft.Initialize();
	m_TimeLeft.SetOrigin(0, 30);
	m_TimeLeft.SetSize(40, 12);
	m_TimeLeft.SetFontFaceName(_T("Small Fonts"));
	m_TimeLeft.SetFontPointSize(6);
	m_TimeLeft.SetAlignment(DT_LEFT);
	m_TimeLeft.SetWordWrap(FALSE);
	m_TimeLeft.SetText(_T(" "));

	m_TimeRight.Initialize();
	m_TimeRight.SetOrigin(0, 30);
	m_TimeRight.SetSize(160, 12);
	m_TimeRight.SetFontFaceName(_T("Small Fonts"));
	m_TimeRight.SetFontPointSize(6);
	m_TimeRight.SetAlignment(DT_RIGHT);
	m_TimeRight.SetWordWrap(FALSE);	
	m_TimeRight.SetText(_T(" "));

	m_TitleLogo.Initialize();
	m_TitleLogo.SetOrigin(5,0);
	m_TitleLogo.SetSize(26,41);
	m_TitleLogo.Show(TRUE);
	m_TitleLogo.SetBitmap((HBITMAP)LoadImage(core_api::get_my_instance(),
							MAKEINTRESOURCE(IDB_LOGO1), IMAGE_BITMAP,
							0, 0, LR_MONOCHROME));

	m_TitleTextLine1.Initialize();
	m_TitleTextLine1.SetOrigin(30, 6);
	m_TitleTextLine1.SetSize(130, 18);
	m_TitleTextLine1.SetFontFaceName(_T("Tahoma"));
	m_TitleTextLine1.SetFontPointSize(10);
	m_TitleTextLine1.SetAlignment(DT_CENTER);
	m_TitleTextLine1.SetWordWrap(FALSE);	
	m_TitleTextLine1.SetText(_T("Foobar2000"));

	m_TitleTextLine2.Initialize();
	m_TitleTextLine2.SetOrigin(30, 20);
	m_TitleTextLine2.SetSize(130, 18);
	m_TitleTextLine2.SetFontFaceName(_T("Tahoma"));
	m_TitleTextLine2.SetFontPointSize(10);
	m_TitleTextLine2.SetAlignment(DT_CENTER);
	m_TitleTextLine2.SetWordWrap(FALSE);	
	m_TitleTextLine2.SetText(_T("Media LCD Display"));

	AddObject(&m_TitleLogo);
	AddObject(&m_TitleTextLine1);
	AddObject(&m_TitleTextLine2);

	return CLCDManager::Initialize();
}

void LCDPlaybackManager::SetTimeFormats(int iLeft, int iRight)
{
	iLeftTimeFormat = iLeft;
	iRightTimeFormat = iRight;
}

void LCDPlaybackManager::SwitchToSongView()
{
	RemoveObject(&m_TitleLogo);
	RemoveObject(&m_TitleTextLine1);
	RemoveObject(&m_TitleTextLine2);

	AddObject(&m_Line1);
	AddObject(&m_Line2);
	AddObject(&m_Line3);
	AddObject(&m_TimeLeft);
	AddObject(&m_TimeRight);
	AddObject(&m_ProgressBar);
}


void LCDPlaybackManager::UpdateLines(pfc::string8 sLine1, pfc::string8 sLine2, pfc::string8 sLine3)
{
    pfc::stringcvt::string_os_from_utf8 cLine1(sLine1);
	pfc::stringcvt::string_os_from_utf8 cLine2(sLine2);
	pfc::stringcvt::string_os_from_utf8 cLine3(sLine3);
    m_Line1.SetText(cLine1);
	m_Line2.SetText(cLine2);
	m_Line3.SetText(cLine3);
}

void LCDPlaybackManager::SetTrackLength(double dTime)
{
	if(dTime < 0)
	{
		strcpy(m_szTrackLength,"?:??");	
	}
	else
	{
		char szSeconds[10];
		char szMinutes[10];
		char szTime[255];

		// Format Length
		int iTime = (int)dTime;
		int iSeconds = iTime%60;
		int iMinutes = (iTime - iSeconds)/60;
		
		_itoa(iSeconds, szSeconds, 10);		
		_itoa(iMinutes, szMinutes, 10);		
		
		strcpy(szTime,szMinutes);
		strcat(szTime,":");
		if(iSeconds < 10) strcat(szTime,"0");
		strcat(szTime,szSeconds);
		strcpy(m_szTrackLength, szTime);

		m_ProgressBar.SetRange(0, (int)dTime);
	}
	m_dTrackLength = dTime;

	UpdateTrackTimeText();
}


void LCDPlaybackManager::SetTrackPosition(double dTime)
{
	char szSeconds[10];
	char szMinutes[10];
	char szTime[255];

	// Format Current Position
	int iTime = (int)dTime;
	int iSeconds = iTime%60;
	int iMinutes = (iTime - iSeconds)/60;
	
	_itoa(iSeconds, szSeconds, 10);	
	_itoa(iMinutes, szMinutes, 10);	
	
	strcpy(szTime,szMinutes);
	strcat(szTime,":");
	if(iSeconds < 10) strcat(szTime,"0");
	strcat(szTime,szSeconds);
	strcpy(m_szTrackPosition, szTime);

	// Format Countdown
	if(m_dTrackLength < 0)
	{
		strcpy(m_szTrackCountDown,"?:??");	
	}
	else
	{
		int iTime = (int)(m_dTrackLength - dTime);
		int iSeconds = iTime%60;
		int iMinutes = (iTime - iSeconds)/60;
		
		_itoa(iSeconds, szSeconds, 10);	
		_itoa(iMinutes, szMinutes, 10);	
		
		strcpy(szTime, "-");
		strcat(szTime, szMinutes);
		strcat(szTime,":");
		if(iSeconds < 10) strcat(szTime,"0");
		strcat(szTime,szSeconds);
		strcpy(m_szTrackCountDown, szTime);
	}

	if(m_dTrackLength < 0) m_ProgressBar.SetPos(0);
	else m_ProgressBar.SetPos((float)dTime);

	UpdateTrackTimeText();
}

void LCDPlaybackManager::UpdateTrackTimeText()
{
	switch(iLeftTimeFormat)
	{
		case TIME_CURRENT_POS:
		{
			pfc::stringcvt::string_os_from_utf8 cTime(m_szTrackPosition);
			m_TimeLeft.SetText(cTime);
		}
		break;

		case TIME_TRACK_LENGTH:
		{
			pfc::stringcvt::string_os_from_utf8 cTime(m_szTrackLength);
			m_TimeLeft.SetText(cTime);
		}
		break;

		case TIME_COUNTDOWN:
		{
			pfc::stringcvt::string_os_from_utf8 cTime(m_szTrackCountDown);
			m_TimeLeft.SetText(cTime);
		}
		break;
	}

	switch(iRightTimeFormat)
	{
		case TIME_CURRENT_POS:
		{
			pfc::stringcvt::string_os_from_utf8 cTime(m_szTrackPosition);
			m_TimeRight.SetText(cTime);
		}
		break;

		case TIME_TRACK_LENGTH:
		{
			pfc::stringcvt::string_os_from_utf8 cTime(m_szTrackLength);
			m_TimeRight.SetText(cTime);
		}
		break;

		case TIME_COUNTDOWN:
		{
			pfc::stringcvt::string_os_from_utf8 cTime(m_szTrackCountDown);
			m_TimeRight.SetText(cTime);
		}
		break;
	}
}

void LCDPlaybackManager::OnLCDButtonDown(int nButton)
{
	switch(nButton)
    {
    case LGLCDBUTTON_BUTTON0:
        SetActiveTitleFormat(0);
        break;
    case LGLCDBUTTON_BUTTON1:
        SetActiveTitleFormat(1);
        break;
    case LGLCDBUTTON_BUTTON2:
        SetActiveTitleFormat(2);
        break;
    case LGLCDBUTTON_BUTTON3:
        SetActiveTitleFormat(3);
        break;
    default:
        break;
    }
}