#include "LCDPlayback.h"
#include "resource.h"

DECLARE_COMPONENT_VERSION(COMPONENT_NAME, COMPONENT_VERSION, COMPONENT_DESCRIPTION);

static LCD_Playback g_PlaybackObj;
char g_szTemp[255];
double g_dTemp;

// Priority Setting Variables
static const GUID guid_cfg_lcd_set_priority = { 0xe7f1fd8a, 0xb9f6, 0x4565, { 0x84, 0x81, 0x90, 0xa1, 0xf, 0x71, 0xea, 0x92 } };
static cfg_bool cfg_lcd_set_priority(guid_cfg_lcd_set_priority, false);
static const GUID guid_cfg_lcd_priority_time = { 0x3b9d9fdb, 0xb584, 0x4bf9, { 0x8f, 0x1f, 0x73, 0xee, 0xf4, 0xbd, 0x41, 0x49 } };
static cfg_int cfg_lcd_priority_time(guid_cfg_lcd_priority_time, 5);

// Track Time Variables
static const GUID guid_cfg_lcd_time_left = { 0xecb82ff1, 0xd4cd, 0x422e, { 0x84, 0x5e, 0xa2, 0x9e, 0x3a, 0x81, 0x24, 0x25 } };
static cfg_int cfg_lcd_time_left(guid_cfg_lcd_time_left, TIME_CURRENT_POS);
static const GUID guid_cfg_lcd_time_right = { 0xb8e3696a, 0x7203, 0x4177, { 0x8e, 0x16, 0x34, 0x8f, 0x6e, 0xe4, 0x2e, 0x22 } };
static cfg_int cfg_lcd_time_right(guid_cfg_lcd_time_right, TIME_TRACK_LENGTH);

// Softkey 1 Config Variables
static const GUID guid_cfg_lcd_pref_sk1line1 = { 0x21804e0d, 0x329a, 0x4f01, { 0x9b, 0x7a, 0xfc, 0x7d, 0xb8, 0x8f, 0xc2, 0x1c } };
static cfg_string cfg_lcd_pref_sk1line1(guid_cfg_lcd_pref_sk1line1, "%title%");
static const GUID guid_cfg_lcd_pref_sk1line2 = { 0xd3300137, 0xe75c, 0x4647, { 0xa4, 0x7a, 0xaa, 0x2a, 0x45, 0xb, 0x43, 0xac } };
static cfg_string cfg_lcd_pref_sk1line2(guid_cfg_lcd_pref_sk1line2, "%artist%");
static const GUID guid_cfg_lcd_pref_sk1line3 = { 0xa3ee5698, 0xf656, 0x41d8, { 0xbd, 0x68, 0x94, 0x11, 0xb5, 0x4, 0x24, 0x40 } };
static cfg_string cfg_lcd_pref_sk1line3(guid_cfg_lcd_pref_sk1line3, "%album% '('%date%')'");

// Softkey 2 Config Variables
static const GUID guid_cfg_lcd_pref_sk2line1 = { 0x27bdb988, 0xc8d1, 0x46bc, { 0xaf, 0x8a, 0xf9, 0xda, 0x78, 0xee, 0x75, 0x6f } };
static cfg_string cfg_lcd_pref_sk2line1(guid_cfg_lcd_pref_sk2line1, "%artist%");
static const GUID guid_cfg_lcd_pref_sk2line2 = { 0x32f6efcc, 0x98ec, 0x49e2, { 0x8d, 0x1b, 0xcc, 0x22, 0x9, 0xd2, 0xf8, 0x33 } };
static cfg_string cfg_lcd_pref_sk2line2(guid_cfg_lcd_pref_sk2line2, "%title%");
static const GUID guid_cfg_lcd_pref_sk2line3 = { 0x10982d61, 0xa51b, 0x48f0, { 0x9b, 0x2f, 0xff, 0xf6, 0xaa, 0x69, 0xd, 0x64 } };
static cfg_string cfg_lcd_pref_sk2line3(guid_cfg_lcd_pref_sk2line3, "%album%");

// Softkey 3 Config Variables
static const GUID guid_cfg_lcd_pref_sk3line1 = { 0x7642faa4, 0x9c30, 0x4d48, { 0x8a, 0x3d, 0x22, 0xd2, 0xfc, 0xdb, 0x23, 0xf5 } };
static cfg_string cfg_lcd_pref_sk3line1(guid_cfg_lcd_pref_sk3line1, "%title%");
static const GUID guid_cfg_lcd_pref_sk3line2 = { 0x4de2ee1c, 0x63e8, 0x4cad, { 0x88, 0x4f, 0x73, 0x2a, 0x81, 0x57, 0x61, 0x6a } };
static cfg_string cfg_lcd_pref_sk3line2(guid_cfg_lcd_pref_sk3line2, "%artist%");
static const GUID guid_cfg_lcd_pref_sk3line3 = { 0xefb24e6e, 0xe2b2, 0x42f8, { 0xbe, 0x2b, 0x2a, 0x8b, 0xee, 0xdf, 0x26, 0xdd } };
static cfg_string cfg_lcd_pref_sk3line3(guid_cfg_lcd_pref_sk3line3, "");

// Softkey 4 Config Variables
static const GUID guid_cfg_lcd_pref_sk4line1 = { 0xf00c97df, 0x9225, 0x4fd1, { 0xb8, 0x47, 0x9f, 0xd9, 0x79, 0x6b, 0x2, 0x3 } };
static cfg_string cfg_lcd_pref_sk4line1(guid_cfg_lcd_pref_sk4line1, "%artist% - %title%");
static const GUID guid_cfg_lcd_pref_sk4line2 = { 0x7cbd05d7, 0x7618, 0x4615, { 0xba, 0x67, 0xbb, 0x49, 0x23, 0xb, 0x58, 0xea } };
static cfg_string cfg_lcd_pref_sk4line2(guid_cfg_lcd_pref_sk4line2, "%album% '('%date%')'");
static const GUID guid_cfg_lcd_pref_sk4line3 = { 0x99986f9e, 0x5585, 0x4e7b, { 0xb5, 0xe7, 0xa6, 0x61, 0x68, 0x13, 0x2c, 0xbf } };
static cfg_string cfg_lcd_pref_sk4line3(guid_cfg_lcd_pref_sk4line3, "%__mp3_stereo_mode%");

void SetStringsInPlayback()
{
	pfc::string8 tempString[12];
	tempString[0] = cfg_lcd_pref_sk1line1;
	tempString[1] = cfg_lcd_pref_sk1line2;
	tempString[2] = cfg_lcd_pref_sk1line3;
	tempString[3] = cfg_lcd_pref_sk2line1;
	tempString[4] = cfg_lcd_pref_sk2line2;
	tempString[5] = cfg_lcd_pref_sk2line3;
	tempString[6] = cfg_lcd_pref_sk3line1;
	tempString[7] = cfg_lcd_pref_sk3line2;
	tempString[8] = cfg_lcd_pref_sk3line3;
	tempString[9] = cfg_lcd_pref_sk4line1;
	tempString[10] = cfg_lcd_pref_sk4line2;
	tempString[11] = cfg_lcd_pref_sk4line3;
	g_PlaybackObj.SetFormatStrings(tempString);
}

void CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime ) 
{
	 g_PlaybackObj.TimerTick(); 
}

void SetActiveTitleFormat(int i)
{
	g_PlaybackObj.SetActiveFormat(i);
}

class initquit_g15lcd : public initquit {
	virtual void on_init() 
	{
		SetStringsInPlayback();
		g_PlaybackObj.SetTimeFormats(cfg_lcd_time_left.get_value(), cfg_lcd_time_right.get_value());
		g_PlaybackObj.SetActiveFormat(0);
		g_PlaybackObj.Start();
	}

	virtual void on_quit() 
	{
		g_PlaybackObj.Stop();
	}
};

static initquit_factory_t< initquit_g15lcd > foo_initquit;

static BOOL CALLBACK ConfigProc(HWND wnd,UINT msg,WPARAM wp,LPARAM lp)
{
    switch(msg)
    {
        case WM_INITDIALOG:
		{
            uSetDlgItemText(wnd, IDC_KEY1LINE1, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk1line1));
            uSetDlgItemText(wnd, IDC_KEY1LINE2, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk1line2));
            uSetDlgItemText(wnd, IDC_KEY1LINE3, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk1line3));
            uSetDlgItemText(wnd, IDC_KEY2LINE1, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk2line1));
            uSetDlgItemText(wnd, IDC_KEY2LINE2, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk2line2));
            uSetDlgItemText(wnd, IDC_KEY2LINE3, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk2line3));
            uSetDlgItemText(wnd, IDC_KEY3LINE1, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk3line1));
            uSetDlgItemText(wnd, IDC_KEY3LINE2, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk3line2));
            uSetDlgItemText(wnd, IDC_KEY3LINE3, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk3line3));
            uSetDlgItemText(wnd, IDC_KEY4LINE1, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk4line1));
            uSetDlgItemText(wnd, IDC_KEY4LINE2, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk4line2));
            uSetDlgItemText(wnd, IDC_KEY4LINE3, pfc::stringcvt::string_ansi_from_utf8(cfg_lcd_pref_sk4line3));
			switch(cfg_lcd_time_left.get_value())
			{
				case TIME_CURRENT_POS: 
					CheckRadioButton(wnd, IDC_LEFTPOS, IDC_LEFTCD, IDC_LEFTPOS);
					break;

				case TIME_TRACK_LENGTH: 
					CheckRadioButton(wnd, IDC_LEFTPOS, IDC_LEFTCD, IDC_LEFTLENGTH);
					break;

				case TIME_COUNTDOWN: 
					CheckRadioButton(wnd, IDC_LEFTPOS, IDC_LEFTCD, IDC_LEFTCD);
					break;
			}
			switch(cfg_lcd_time_right.get_value())
			{
				case TIME_CURRENT_POS: 
					CheckRadioButton(wnd, IDC_RIGHTPOS, IDC_RIGHTCD, IDC_RIGHTPOS);
					break;

				case TIME_TRACK_LENGTH: 
					CheckRadioButton(wnd, IDC_RIGHTPOS, IDC_RIGHTCD, IDC_RIGHTLENGTH);
					break;

				case TIME_COUNTDOWN: 
					CheckRadioButton(wnd, IDC_RIGHTPOS, IDC_RIGHTCD, IDC_RIGHTCD);
					break;
			}
			if(cfg_lcd_set_priority.get_value())
			{
				CheckDlgButton(wnd, IDC_PRIORITY, true);
				EnableWindow(GetDlgItem(wnd,IDC_PRIORITYTIME),true);
			}
			else
			{
				CheckDlgButton(wnd, IDC_PRIORITY, false);
				EnableWindow(GetDlgItem(wnd,IDC_PRIORITYTIME),false);
			}
			pfc::float_to_string(g_szTemp,5,(double)(cfg_lcd_priority_time.get_value()),0);
			pfc::stringcvt::string_os_from_utf8 cTime(g_szTemp);
			g_PlaybackObj.SetPrioritySettings(cfg_lcd_set_priority.get_value(), cfg_lcd_priority_time.get_value());
			SetDlgItemText(wnd,IDC_PRIORITYTIME,cTime);
		}
	    break;

        case WM_COMMAND:
            switch (wp)
			{
				case IDC_PRIORITY:
					if(IsDlgButtonChecked(wnd, IDC_PRIORITY))
					{
						cfg_lcd_set_priority = true;
						EnableWindow(GetDlgItem(wnd,IDC_PRIORITYTIME),true);
						g_PlaybackObj.SetPrioritySettings(cfg_lcd_set_priority.get_value(), cfg_lcd_priority_time.get_value());
					}
					else
					{
						cfg_lcd_set_priority = false;
						EnableWindow(GetDlgItem(wnd,IDC_PRIORITYTIME),false);
						g_PlaybackObj.SetPrioritySettings(cfg_lcd_set_priority.get_value(), cfg_lcd_priority_time.get_value());
					}
					break;

				case IDC_PRIORITYTIME | (EN_KILLFOCUS << 16):
					{											
						pfc::string8 strTemp;
						strTemp = string_utf8_from_window(wnd, IDC_PRIORITYTIME);
						g_dTemp = pfc::string_to_float(strTemp);
						if(g_dTemp > 30) g_dTemp = 30;
						if(g_dTemp < 1) g_dTemp = 1;
						pfc::float_to_string(g_szTemp,5,g_dTemp,0);
						pfc::stringcvt::string_os_from_utf8 cTime(g_szTemp);
						cfg_lcd_priority_time = (int)g_dTemp;
						SetDlgItemText(wnd,IDC_PRIORITYTIME,cTime);
						g_PlaybackObj.SetPrioritySettings(cfg_lcd_set_priority.get_value(), cfg_lcd_priority_time.get_value());
					}
					break;

				case IDC_LEFTPOS:
					cfg_lcd_time_left = TIME_CURRENT_POS;
					g_PlaybackObj.SetTimeFormats(cfg_lcd_time_left.get_value(), cfg_lcd_time_right.get_value());
					break;

				case IDC_LEFTLENGTH:
					cfg_lcd_time_left = TIME_TRACK_LENGTH;
					g_PlaybackObj.SetTimeFormats(cfg_lcd_time_left.get_value(), cfg_lcd_time_right.get_value());
					break;

				case IDC_LEFTCD:
					cfg_lcd_time_left = TIME_COUNTDOWN;
					g_PlaybackObj.SetTimeFormats(cfg_lcd_time_left.get_value(), cfg_lcd_time_right.get_value());
					break;

				case IDC_RIGHTPOS:
					cfg_lcd_time_right = TIME_CURRENT_POS;
					g_PlaybackObj.SetTimeFormats(cfg_lcd_time_left.get_value(), cfg_lcd_time_right.get_value());
					break;

				case IDC_RIGHTLENGTH:
					cfg_lcd_time_right = TIME_TRACK_LENGTH;
					g_PlaybackObj.SetTimeFormats(cfg_lcd_time_left.get_value(), cfg_lcd_time_right.get_value());
					break;

				case IDC_RIGHTCD:
					cfg_lcd_time_right = TIME_COUNTDOWN;
					g_PlaybackObj.SetTimeFormats(cfg_lcd_time_left.get_value(), cfg_lcd_time_right.get_value());
					break;

			    case IDC_KEY1LINE1 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk1line1 = string_utf8_from_window(wnd, IDC_KEY1LINE1);
					SetStringsInPlayback();
					break;

			    case IDC_KEY1LINE2 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk1line2 = string_utf8_from_window(wnd, IDC_KEY1LINE2);
					SetStringsInPlayback();
					break;

			    case IDC_KEY1LINE3 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk1line3 = string_utf8_from_window(wnd, IDC_KEY1LINE3);
					SetStringsInPlayback();
					break;

			    case IDC_KEY2LINE1 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk2line1 = string_utf8_from_window(wnd, IDC_KEY2LINE1);
					SetStringsInPlayback();
					break;

			    case IDC_KEY2LINE2 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk2line2 = string_utf8_from_window(wnd, IDC_KEY2LINE2);
					SetStringsInPlayback();
					break;

			    case IDC_KEY2LINE3 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk2line3 = string_utf8_from_window(wnd, IDC_KEY2LINE3);
					SetStringsInPlayback();
					break;

			    case IDC_KEY3LINE1 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk3line1 = string_utf8_from_window(wnd, IDC_KEY3LINE1);
					SetStringsInPlayback();
					break;

			    case IDC_KEY3LINE2 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk3line2 = string_utf8_from_window(wnd, IDC_KEY3LINE2);
					SetStringsInPlayback();
					break;

			    case IDC_KEY3LINE3 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk3line3 = string_utf8_from_window(wnd, IDC_KEY3LINE3);
					SetStringsInPlayback();
					break;

			    case IDC_KEY4LINE1 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk4line1 = string_utf8_from_window(wnd, IDC_KEY4LINE1);
					SetStringsInPlayback();
					break;

			    case IDC_KEY4LINE2 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk4line2 = string_utf8_from_window(wnd, IDC_KEY4LINE2);
					SetStringsInPlayback();
					break;

			    case IDC_KEY4LINE3 | (EN_CHANGE << 16):
					cfg_lcd_pref_sk4line3 = string_utf8_from_window(wnd, IDC_KEY4LINE3);
					SetStringsInPlayback();
					break;
			}
		    break;
        case WM_DESTROY:
	        break;
        
        default:
            return FALSE;
    }
    return 0;
}

static const GUID guid_cfg_lcd_pref_dialog = { 0x579306a7, 0x3d5f, 0x4b2d, { 0x8a, 0x5d, 0x56, 0x35, 0x46, 0x95, 0xa7, 0xcd } };

class preferences_page_g15lcd : public preferences_page
{
public:
	virtual const char * get_name() { return "Logitech G15 LCD"; }
	virtual GUID get_guid()	{ return guid_cfg_lcd_pref_dialog;	}
	virtual GUID get_parent_guid() { return preferences_page::guid_display;	}
	virtual bool reset_query() { return false; }
	virtual void reset() { }
	virtual HWND create(HWND parent) { return uCreateDialog(IDD_LCDPREFS, parent, ConfigProc, 0); }
};

static preferences_page_factory_t<preferences_page_g15lcd> foo_preferences_page_g15lcd;
